-- OmniStats
-- Provides character attributes in a compact frame
-- Author: stjobe

local OmniStats = {}

OmniStats.Defaults = {
	posX = 30,
	posY = 100,
	ShowMax = false,
	Layout = "Rectangle",
	BDAlpha = 0,
	TextAlpha = 1,
	Scale = 1,
	ShowIC = true,
	ShowOOC = true,
	Manual = false,
}

OmniStats.RefStatDefaults = {[4] = 0, [5] = 0, [7] = 0, [8] = 0, [29] = 0, [30] = 0, [25] = 0, [13] = 0, [23] = 0, [35] = 0, [16] = 0, [22] = 0}

OmniStats.Labels = {
	[4] = "Mag: ",
	[5] = "MaR: ",
	[7] = "Hth: ",
	[8] = "HtR: ",
	[29] = "Sta: ",
	[30] = "StR: ",
	[25] = "SpD: ",
	[13] = "SpR: ",
	[23] = "SpC: ",
	[35] = "Wpn: ",
	[16] = "WpC: ",	
	[22] = "Arm: "
}

OmniStats.CurrStats = {}
OmniStats.NameControls = {}
OmniStats.ValueControls = {}
OmniStats.IconControls = {}

function OmniStats.GetBaseValues()
    for stat, _ in pairs(OmniStats.Labels) do
		OmniStats.RefStats[stat] = GetPlayerStat(stat, 0, 0)
	end
end

function OmniStats.GetValues()
    for stat, _ in pairs(OmniStats.Labels) do
		if (OmniStats.SV.ShowMax == false) then
			if (stat == 4) then
				OmniStats.CurrStats[stat], _, _ = GetUnitPower("player", POWERTYPE_MAGICKA)
			elseif (stat == 7) then
				OmniStats.CurrStats[stat], _, _ = GetUnitPower("player", POWERTYPE_HEALTH)
			elseif (stat == 29) then
				OmniStats.CurrStats[stat], _, _ = GetUnitPower("player", POWERTYPE_STAMINA)
			else
				OmniStats.CurrStats[stat] = GetPlayerStat(stat, 0, 0)
			end
		else -- always show max
			if (stat == 4) then
				_, _, OmniStats.CurrStats[stat] = GetUnitPower("player", POWERTYPE_MAGICKA)
			elseif (stat == 7) then
				_, _, OmniStats.CurrStats[stat] = GetUnitPower("player", POWERTYPE_HEALTH)
			elseif (stat == 29) then
				_, _, OmniStats.CurrStats[stat] = GetUnitPower("player", POWERTYPE_STAMINA)
			else
				OmniStats.CurrStats[stat] = GetPlayerStat(stat, 0, 0)
			end
		end
    end
end

function OmniStats.UpdateUI()
    for stat, value in pairs(OmniStats.CurrStats) do
		OmniStats.ValueControls[stat]:SetColor(1,1,1,1)
		local softcap = GetStatSoftCap(stat)
		if softcap == nil then 
			softcap = 10000 -- workaround for Wpn, WpC, SpC 
		end
		if (value > softcap) then -- softcapped
			OmniStats.ValueControls[stat]:SetColor(0.9,0.5,0.3,1)
			OmniStats.IconControls[stat]:SetHidden(false)
		else
			OmniStats.IconControls[stat]:SetHidden(true)
		end
		if value > OmniStats.RefStats[stat] then -- buffed
            OmniStats.ValueControls[stat]:SetColor(0,1,0,1)
        elseif value < OmniStats.RefStats[stat] then -- debuffed
            OmniStats.ValueControls[stat]:SetColor(1,0,0,1)
        end
		if (stat == 16 or stat == 23) then
			OmniStats.ValueControls[stat]:SetText(string.format("%.0f", OmniStats.CurrStats[stat] / 10).."%")
		else
			OmniStats.ValueControls[stat]:SetText(string.format("%d", OmniStats.CurrStats[stat]))
		end
    end
end

function OmniStats.CreateLayout()
	local offset
	if (OmniStats.SV.Layout == "Horizontal") then
		for index, value in next, {4,7,29,5,8,30,25,23,13,35,16,22} do
			offset = index - 1
			OmniStats.NameControls[value]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, offset * 70, 0)
			offset = offset + 1
		end
		OmniStatsMainWindow:SetDimensions(840, 16)
	elseif (OmniStats.SV.Layout == "Vertical") then
		for index, value in next, {4,7,29,5,8,30,25,23,13,35,16,22} do
			offset = index - 1
			OmniStats.NameControls[value]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, offset * 16)
			offset = offset + 1
		end
		OmniStatsMainWindow:SetDimensions(70, 192)
	elseif (OmniStats.SV.Layout == "Square") then
		OmniStats.NameControls[4]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 0)
		OmniStats.NameControls[7]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 16)
		OmniStats.NameControls[29]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 32)
		OmniStats.NameControls[25]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 48)
		OmniStats.NameControls[23]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 64)
		OmniStats.NameControls[13]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 80)
		
		OmniStats.NameControls[5]:SetAnchor(TOPLEFT, OmniStats.IconControls[4], TOPRIGHT, 5, 0)
		OmniStats.NameControls[8]:SetAnchor(TOPLEFT, OmniStats.IconControls[7], TOPRIGHT, 5, 0)
		OmniStats.NameControls[30]:SetAnchor(TOPLEFT, OmniStats.IconControls[29], TOPRIGHT, 5, 0)
		OmniStats.NameControls[35]:SetAnchor(TOPLEFT, OmniStats.IconControls[25], TOPRIGHT, 5, 0)
		OmniStats.NameControls[16]:SetAnchor(TOPLEFT, OmniStats.IconControls[23], TOPRIGHT, 5, 0)
		OmniStats.NameControls[22]:SetAnchor(TOPLEFT, OmniStats.IconControls[13], TOPRIGHT, 5, 0)
		OmniStatsMainWindow:SetDimensions(140, 96)
	else
		OmniStats.NameControls[4]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 0)
		OmniStats.NameControls[7]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 16)
		OmniStats.NameControls[29]:SetAnchor(TOPLEFT, OmniStatsMainWindow, TOPLEFT, 0, 32)
		
		OmniStats.NameControls[5]:SetAnchor(TOPLEFT, OmniStats.IconControls[4], TOPRIGHT, 5, 0)
		OmniStats.NameControls[8]:SetAnchor(TOPLEFT, OmniStats.IconControls[7], TOPRIGHT, 5, 0)
		OmniStats.NameControls[30]:SetAnchor(TOPLEFT, OmniStats.IconControls[29], TOPRIGHT, 5, 0)
		
		OmniStats.NameControls[25]:SetAnchor(TOPLEFT, OmniStats.IconControls[5], TOPRIGHT, 5, 0)
		OmniStats.NameControls[23]:SetAnchor(TOPLEFT, OmniStats.IconControls[8], TOPRIGHT, 5, 0)
		OmniStats.NameControls[13]:SetAnchor(TOPLEFT, OmniStats.IconControls[30], TOPRIGHT, 5, 0)
		
		OmniStats.NameControls[35]:SetAnchor(TOPLEFT, OmniStats.IconControls[25], TOPRIGHT, 5, 0)
		OmniStats.NameControls[16]:SetAnchor(TOPLEFT, OmniStats.IconControls[23], TOPRIGHT, 5, 0)
		OmniStats.NameControls[22]:SetAnchor(TOPLEFT, OmniStats.IconControls[13], TOPRIGHT, 5, 0)
		OmniStatsMainWindow:SetDimensions(280, 48)
	end
end

function OmniStats.CreateUI()
	OmniStats.MainWindow = WINDOW_MANAGER:CreateTopLevelWindow("OmniStatsMainWindow")
    OmniStatsMainWindow:SetDimensions(280, 48)
    OmniStatsMainWindow:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, OmniStats.SV.posX, OmniStats.SV.posY)
    OmniStatsMainWindow:SetHidden(false)
    OmniStatsMainWindow:SetMovable(true)
    OmniStatsMainWindow:SetMouseEnabled(true)
	OmniStatsMainWindow:SetHandler("OnMouseUp", function(_, button)
			if button == 2 then
				OmniStats.GetBaseValues()
			elseif button == 1 then
				OmniStats.SV.posX = math.floor(OmniStatsMainWindow:GetLeft())
				OmniStats.SV.posY = math.floor(OmniStatsMainWindow:GetTop())
			end
		end)
	OmniStatsMainWindow:SetAlpha(OmniStats.SV.TextAlpha)

	-- controls
    for stat, _ in pairs(OmniStats.Labels) do
		OmniStats.NameControls[stat] = WINDOW_MANAGER:CreateControl("OmniStatsName"..stat, OmniStats.MainWindow, CT_LABEL)
        OmniStats.NameControls[stat]:SetHidden(false)
        OmniStats.NameControls[stat]:SetDimensions(30,16)
        OmniStats.NameControls[stat]:SetAlpha(1)
		OmniStats.NameControls[stat]:SetFont("ZoFontGameSmall")
        OmniStats.NameControls[stat]:SetColor(1,0.98,0.8,1)
        OmniStats.NameControls[stat]:SetText(string.format("%s", OmniStats.Labels[stat]))
		
		OmniStats.ValueControls[stat] = WINDOW_MANAGER:CreateControl("OmniStatsValue"..stat, OmniStats.MainWindow, CT_LABEL)
		OmniStats.ValueControls[stat]:SetHidden(false)
        OmniStats.ValueControls[stat]:SetFont("ZoFontGameSmall")
        OmniStats.ValueControls[stat]:SetDimensions(24,16)
        OmniStats.ValueControls[stat]:SetColor(1,1,1,1)
        OmniStats.ValueControls[stat]:SetAlpha(1)
        OmniStats.ValueControls[stat]:SetAnchor(TOPLEFT, OmniStats.NameControls[stat], TOPRIGHT, 0,0)
		OmniStats.ValueControls[stat]:SetHorizontalAlignment(2) -- align right
        OmniStats.ValueControls[stat]:SetText("0")
		
		OmniStats.IconControls[stat] = WINDOW_MANAGER:CreateControl("OmniStatsIcon"..stat, OmniStats.MainWindow, CT_TEXTURE)
		OmniStats.IconControls[stat]:SetHidden(true)
		OmniStats.IconControls[stat]:SetDimensions(16,16)
		OmniStats.IconControls[stat]:SetAlpha(1)
		OmniStats.IconControls[stat]:SetAnchor(TOPLEFT, OmniStats.ValueControls[stat], TOPRIGHT, 0, 0)
		OmniStats.IconControls[stat]:SetTexture("ESOUI/art/stats/diminishingreturns_icon.dds")
		OmniStats.IconControls[stat]:SetDrawLevel(1)
    end
	
	-- Layout
	OmniStats.CreateLayout()
	
	-- Backdrop
	OmniStats.MainBD = WINDOW_MANAGER:CreateControlFromVirtual("OmniStatsMainBD", OmniStats.MainWindow, "ZO_DefaultBackdrop")
	OmniStatsMainBD:SetAlpha(OmniStats.SV.BDAlpha)

	-- Set scale
	OmniStatsMainWindow:SetScale(OmniStats.SV.Scale)
end

function OmniStats.GetTextAlpha()
	return OmniStats.SV.TextAlpha * 100
end

function OmniStats.SetTextAlpha(newValue)
	if newValue ~= OmniStats.SV.TextAlpha then
		OmniStats.SV.TextAlpha = newValue / 100
		OmniStatsMainWindow:SetAlpha(OmniStats.SV.TextAlpha)
	end
end

function OmniStats.GetBDAlpha()
	return OmniStats.SV.BDAlpha * 100
end

function OmniStats.SetBDAlpha(newValue)
	if newValue ~= OmniStats.SV.BDAlpha then
		OmniStats.SV.BDAlpha = newValue / 100
		OmniStatsMainBD:SetAlpha(OmniStats.SV.BDAlpha)
	end
end

function OmniStats.GetScale()
	return OmniStats.SV.Scale * 100
end

function OmniStats.SetScale(newValue)
	if newValue ~= OmniStats.SV.Scale then
		OmniStats.SV.Scale = newValue / 100
		OmniStatsMainWindow:SetScale(OmniStats.SV.Scale)
	end
end

function OmniStats.GetLayout()
	return OmniStats.SV.Layout
end

function OmniStats.SetLayout(newValue)
	if newValue ~= OmniStats.SV.Layout then
		OmniStats.SV.Layout = newValue
		OmniStats.CreateLayout()
	end
end

function OmniStats.GetShowMax()
	return OmniStats.SV.ShowMax
end

function OmniStats.SetShowMax(newValue)
	OmniStats.SV.ShowMax = newValue
end

function OmniStats.GetShowIC()
	return OmniStats.SV.ShowIC
end

function OmniStats.SetShowIC(newValue)
	OmniStats.SV.ShowIC = newValue
	if newValue == true then
		OmniStats.SetManual(false)
	end
end

function OmniStats.GetShowOOC()
	return OmniStats.SV.ShowOOC
end

function OmniStats.SetShowOOC(newValue)
	OmniStats.SV.ShowOOC = newValue
	if newValue == true then
		OmniStats.SetManual(false)
	end
end

function OmniStats.GetManual()
	return OmniStats.SV.Manual
end

function OmniStats.SetManual(newValue)
	OmniStats.SV.Manual = newValue
	if newValue == true then
		OmniStats.SetShowIC(false)
		OmniStats.SetShowOOC(false)
	end
end

function OmniStats.CreateSettings()
	local LAM = LibStub("LibAddonMenu-1.0")
	OmniStats.panelID = LAM:CreateControlPanel("OmniStatsPanel", "OmniStats")
	LAM:AddHeader(OmniStats.panelID, "OmniStatsSettings", "Options")
	LAM:AddCheckbox(OmniStats.panelID, "OmniStatsShowMax", "Always show max values", "Checking this option prevents regular depletion of Magicka, Health, and Stamina to show as debuffed", OmniStats.GetShowMax, OmniStats.SetShowMax)
	LAM:AddDropdown(OmniStats.panelID, "OmniStatsColumns", "Layout", "Sets the layout the stats are displayed with", {"Vertical", "Square", "Rectangle", "Horizontal"}, OmniStats.GetLayout, OmniStats.SetLayout)
	LAM:AddSlider(OmniStats.panelID, "OmniStatsTextAlpha", "Text alpha (percent)", "Sets the text alpha", 0, 100, 1, OmniStats.GetTextAlpha, OmniStats.SetTextAlpha)
	LAM:AddSlider(OmniStats.panelID, "OmniStatsBDAlpha", "Background alpha (percent)", "Sets the background alpha", 0, 100, 1, OmniStats.GetBDAlpha, OmniStats.SetBDAlpha)
	LAM:AddSlider(OmniStats.panelID, "OmniStatsScale", "Window scaling (percent)", "Sets the scaling", 50, 200, 1, OmniStats.GetScale, OmniStats.SetScale)
	LAM:AddCheckbox(OmniStats.panelID, "OmniStatsManual", "Use manual toggle button", "Use a button to toggle OmniStats (disables automatic showing)", OmniStats.GetManual, OmniStats.SetManual)
	LAM:AddCheckbox(OmniStats.panelID, "OmniStatsShowInCombat", "Automatically show in combat", "Check to show stats in combat", OmniStats.GetShowIC, OmniStats.SetShowIC)
	LAM:AddCheckbox(OmniStats.panelID, "OmniStatsShowOutOfCombat", "Automatically show out of combat", "Check to show stats out of combat", OmniStats.GetShowOOC, OmniStats.SetShowOOC)
	LAM:AddDescription(OmniStats.panelID, "OmniStatsNote", "Please note:\nSetting 'Use manual toggle button' to 'ON' will disable 'Automatically show in combat' and 'Automatically show out of combat'.\n\nSetting either of those two options to 'ON' will disable 'Use manual toggle button'.")
end

function OmniStatsToggleWindow()
	if OmniStats.MainWindow:IsHidden() then
		OmniStats.MainWindow:SetHidden(false)
	else
		OmniStats.MainWindow:SetHidden(true)
	end
end

function OmniStats.MainLoop()
	if OmniStats.SV.Manual == false then
		if IsUnitInCombat("player") then
			if OmniStats.SV.ShowIC == false then
				OmniStats.MainWindow:SetHidden(true)
			else
				OmniStats.MainWindow:SetHidden(false)
			end
		else 
			if OmniStats.SV.ShowOOC == false then
				OmniStats.MainWindow:SetHidden(true)
			else
				OmniStats.MainWindow:SetHidden(false)
			end
		end
	end

	OmniStats.GetValues(OmniStats.CurrStats)
    OmniStats.UpdateUI()
    zo_callLater(function() OmniStats.MainLoop() end, 1000)
end

function OmniStats.DelayedInit()
	if (OmniStats.RefStats[4] == 0) then
		OmniStats.GetBaseValues()
	end
	OmniStats.MainLoop()
end

function OmniStats.WaitForPlayerLoad()
	if (OmniStats.PlayerActivated == true) then
		zo_callLater(function() OmniStats.DelayedInit() end, 1000)
	else
		zo_callLater(function() OmniStats.WaitForPlayerLoad() end, 1000)
	end
end

local function OnAddOnLoaded(eventCode, addOnName)
	if (addOnName == "OmniStats") then
		OmniStats.SV = ZO_SavedVars:NewAccountWide("OmniStats_SavedVariables", 1, nil, OmniStats.Defaults)
		OmniStats.RefStats = ZO_SavedVars:New("OmniStats_SavedVariables", 1, nil, OmniStats.RefStatDefaults)
		OmniStats.CreateSettings()
		OmniStats.CreateUI()
		OmniStats.WaitForPlayerLoad()
	end
end

local function OnPlayerActivated()
	OmniStats.PlayerActivated = true
end

function OmniStatsInitialize()
	EVENT_MANAGER:RegisterForEvent("OmniStats", EVENT_PLAYER_ACTIVATED, OnPlayerActivated)
	EVENT_MANAGER:RegisterForEvent("OmniStats", EVENT_ADD_ON_LOADED, OnAddOnLoaded)
end